<?php

namespace Drupal\wordsonline_connector\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\wordsonline_connector\WordsOnlineConst;
use Drupal\wordsonline_connector\WordsOnlineState;
use Drupal\wordsonline_connector\WordsOnlineStatus;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\wordsonline_connector\Plugin\tmgmt\Translator\WordsOnlineTranslator;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Drupal\Core\Extension\ModuleExtensionList;
use Symfony\Component\HttpFoundation\RedirectResponse;


/**
 * Controller for handle order.
 */
class LanguageController extends ControllerBase
{
  use StringTranslationTrait;


  /**
   * Database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * ModuleExtensionList.
   *
   * @var Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;


  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   Database connection.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   Messenger.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   File System.
   * @param \Drupal\Core\Extension\ModuleExtensionList
   *   The list module extension list.
   */
  public function __construct(Connection $database, Messenger $messenger, FileSystemInterface $fileSystem, ModuleExtensionList $moduleExtensionList)
  {
    $this->database = $database;
    $this->messenger = $messenger;
    $this->fileSystem = $fileSystem;
    $this->moduleExtensionList = $moduleExtensionList;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    $database = $container->get('database');
    $messenger = $container->get('messenger');
    $fileSystem = $container->get('file_system');
    // $manager = $container->get('plugin.manager.tmgmt_file.format');
    $moduleExtensionList = $container->get('extension.list.module');
    return new static(
      $database,
      $messenger,
      $fileSystem,
      // $manager,
      $moduleExtensionList
    );
  }

  /**
   * Display list wordsonline support language.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function supportLanguages(Request $request)
  {
    $header = [
      "code" => $this->t("Drupal Language Code"),
      "lang_name" => $this->t("Language Name"),
      "wol_code" => $this->t("WordsOnline Language Code")
    ];
    $rows = [];
    $form["#prefix"] =
      "<div><a  href='" . base_path() . WordsOnlineConst::IMPORT_SUPPORT_LANG_URL . "'
         class='wol-btn btn-layout'>Re-import support languages</a>
         </div>";
    $results = $this->database
      ->query(
        WordsOnlineConst::GET_ALL_WORDSONLINE_SUPPORT_LANGUAGES_QUERY
      )
      ->fetchAll();
    foreach ($results as $res) {
      $rows[] = [
        "code" => $res->code,
        "lang_name" => $res->lang_name,
        "wol_code" => $res->wol_code
      ];

    }
    $form["table"] = [
      "#type" => "table",
      "#header" => $header,
      "#rows" => $rows,
      "#empty" => $this->t("No records found"),
      "#attributes" => ["class" => ["wol-table"]],
    ];
    $form["#attached"]["library"][] = "wordsonline_connector/wordsonline_support_language";
    return $form;
  }

  /**
   * reImport list wordsonline support language.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request from client.
   */
  public function importSupportLanguages(Request $request)
  {
    try {
      $npath = $this->moduleExtensionList->getPath(WordsOnlineConst::MODULE_ID);
      $import = $this->fileSystem->realpath($npath . '/js/languages.json');
      $content = file_get_contents($import);
      $langs = json_decode($content);
      foreach ($langs as $lang) {
        $results = $this->database
          ->query(
            WordsOnlineConst::GET_WORDSONLINE_SUPPORT_LANGUAGE_BY_CODE_QUERY,
            [":code" => (string)$lang->code]
          )
          ->fetchAssoc();
        if ($results != NULL) {
          $this->database
            ->update(WordsOnlineConst::LANGUAGE_JOB_TABLE)
            ->condition("code", $lang->code)
            ->fields(
              [
                'code' => $lang->code,
                'lang_name' => $lang->lang_name,
                'wol_code' => $lang->wol_code
              ]
            )
            ->execute();
        } else {
          $this->database->insert(WordsOnlineConst::LANGUAGE_JOB_TABLE)
            ->fields(['code', 'lang_name', 'wol_code'])
            ->values([
              'code' => $lang->code,
              'lang_name' => $lang->lang_name,
              'wol_code' => $lang->wol_code
            ])
            ->execute();
        }

      }
      $this->messenger->addMessage('Re-import supported languages successfully');
    } catch (\Exception $e) {
      $this->messenger->addError($e->getMessage());
    }
    $target_url = base_path() . WordsOnlineConst::SUPPORT_LANG_URL;
    $response = new RedirectResponse($target_url);
    $response->send();
  }

}
